Imports System.ComponentModel
Imports System.Text.RegularExpressions

<ToolboxBitmap(GetType(IntegerRangeProvider), "integer_range_provider.bmp"), _
 ProvideProperty("MinValue", GetType(Control)), _
 ProvideProperty("MaxValue", GetType(Control)), _
 ProvideProperty("RangeFieldName", GetType(Control))> _
Public Class IntegerRangeProvider
    Inherits System.ComponentModel.Component
    Implements IExtenderProvider

#Region " Component Designer generated code "

    Public Sub New(ByVal Container As System.ComponentModel.IContainer)
        MyClass.New()

        'Required for Windows.Forms Class Composition Designer support
        Container.Add(Me)
    End Sub

    Public Sub New()
        MyBase.New()

        'This call is required by the Component Designer.
        InitializeComponent()

        'Add any initialization after the InitializeComponent() call

    End Sub

    'Component overrides dispose to clean up the component list.
    Protected Overloads Overrides Sub Dispose(ByVal disposing As Boolean)
        If disposing Then
            If Not (components Is Nothing) Then
                components.Dispose()
            End If
        End If
        MyBase.Dispose(disposing)
    End Sub

    'Required by the Component Designer
    Private components As System.ComponentModel.IContainer

    'NOTE: The following procedure is required by the Component Designer
    'It can be modified using the Component Designer.
    'Do not modify it using the code editor.
    Friend WithEvents errOutOfRange As System.Windows.Forms.ErrorProvider
    <System.Diagnostics.DebuggerStepThrough()> Private Sub InitializeComponent()
        Me.errOutOfRange = New System.Windows.Forms.ErrorProvider
        '
        'errOutOfRange
        '
        Me.errOutOfRange.BlinkStyle = System.Windows.Forms.ErrorBlinkStyle.NeverBlink

    End Sub

#End Region

    ' Information about a control's range.
    Private Class RangeInfo
        Public MinValue As Integer
        Public MaxValue As Integer
        Public RangeFieldName As String
        
        Public Sub New()
            MinValue = Integer.MinValue
            MaxValue = Integer.MaxValue
            RangeFieldName = Nothing
        End Sub

        ' Return True if this object represents no range.
        Public Function IsDefault() As Boolean
            Return (MinValue = Integer.MinValue) And _
                   (MaxValue = Integer.MaxValue) And _
                   (RangeFieldName Is Nothing)
        End Function
    End Class

    ' The information about fields with ranges.
    Private m_RangeInfo As New Hashtable

    Private m_Enabled As Boolean = True
    <Category("Behavior"), _
     DefaultValue(True)> _
    Public Property Enabled() As Boolean
        Get
            Return m_Enabled
        End Get
        Set(ByVal Value As Boolean)
            m_Enabled = Value
        End Set
    End Property

    ' We can extend TextBoxes.
    Public Function CanExtend(ByVal client_control As Object) As Boolean Implements IExtenderProvider.CanExtend
        Return (TypeOf client_control Is TextBox)
    End Function

    ' Return this control's minimum value.
    <Category("Validation"), _
     DefaultValue(Integer.MinValue)> _
    Public Function GetMinValue(ByVal client_control As Control) As Integer
        Return GetRangeInfo(client_control).MinValue
    End Function

    ' Set this control's minimum value.
    <Category("Validation"), _
     DefaultValue(Integer.MinValue)> _
    Public Sub SetMinValue(ByVal client_control As Control, ByVal min_value As Integer)
        ' Get this control's range info.
        Dim range_info As RangeInfo = GetRangeInfo(client_control)

        ' Set the new minimum value.
        range_info.MinValue = min_value

        ' Add or remove the RangeInfo if necessary.
        AddOrRemoveIfNecessary(client_control, range_info)
    End Sub

    ' Return this control's maximum value.
    <Category("Validation"), _
     DefaultValue(Integer.MaxValue)> _
    Public Function GetMaxValue(ByVal client_control As Control) As Integer
        Return GetRangeInfo(client_control).MaxValue
    End Function

    ' Set this control's maximum value.
    <Category("Validation"), _
     DefaultValue(Integer.MaxValue)> _
    Public Sub SetMaxValue(ByVal client_control As Control, ByVal max_value As Integer)
        ' Get this control's range info.
        Dim range_info As RangeInfo = GetRangeInfo(client_control)

        ' Set the new minimum value.
        range_info.MaxValue = max_value

        ' Add or remove the RangeInfo if necessary.
        AddOrRemoveIfNecessary(client_control, range_info)
    End Sub

    ' Return this control's error message.
    <Category("Validation"), _
     DefaultValue("")> _
    Public Function GetRangeFieldName(ByVal client_control As Control) As String
        Return GetRangeInfo(client_control).RangeFieldName
    End Function

    ' Set this control's error message.
    <Category("Validation"), _
     DefaultValue("")> _
    Public Sub SetRangeFieldName(ByVal client_control As Control, ByVal error_message As String)
        ' Get this control's range info.
        Dim range_info As RangeInfo = GetRangeInfo(client_control)

        ' Set the new minimum value.
        range_info.RangeFieldName = error_message

        ' Add or remove the RangeInfo if necessary.
        AddOrRemoveIfNecessary(client_control, range_info)
    End Sub

    ' A client is validating. See if it is blank.
    Private Sub Client_Validating(ByVal sender As Object, ByVal e As System.ComponentModel.CancelEventArgs)
        If Not m_Enabled Then Exit Sub
        ValidateTextBox(DirectCast(sender, TextBox))
    End Sub

    ' Validate the TextBox.
    Private Sub ValidateTextBox(ByVal client_control As TextBox)
        ' Get the control's text.
        Dim client_text As String = client_control.Text
        Dim range_info As RangeInfo = GetRangeInfo(client_control)

        ' See if the text satisfies the range.
        If client_text.Length = 0 Then
            ' It's blank. Allow it.
            errOutOfRange.SetError(client_control, "")
        Else
            ' See if it looks like an integer.
            Dim r As New Regex("^(-|)[0-9]+$")
            If r.Match(client_text).Success Then
                ' It looks like an integer. Get its value.
                Try
                    Dim client_value As Integer = _
                        Integer.Parse(client_text)
                    If client_value < range_info.MinValue Then
                        ' Too small.
                        errOutOfRange.SetError(client_control, "Field " & range_info.RangeFieldName & " must be no smaller than " & range_info.MinValue.ToString)
                    ElseIf client_value > range_info.MaxValue Then
                        ' Too big.
                        errOutOfRange.SetError(client_control, "Field " & range_info.RangeFieldName & " must be no greater than " & range_info.MaxValue.ToString)
                    Else
                        ' Just right.
                        errOutOfRange.SetError(client_control, "")
                    End If
                Catch ex As Exception
                    ' Something weird is going on. Possibly this
                    ' value has too many digits for an integer.
                    ' Display a generic "bad integer" message.
                    errOutOfRange.SetError(client_control, "Field " & range_info.RangeFieldName & " must be an integer")
                End Try
            Else
                ' It doesn't look like an integer.
                ' Complain.
                errOutOfRange.SetError(client_control, "Field " & range_info.RangeFieldName & " must be an integer")
            End If
        End If
    End Sub

    ' Return this control's RangeInfo.
    Private Function GetRangeInfo(ByVal client_control As Control) As RangeInfo
        ' See if we have RangeInfo for this control.
        If m_RangeInfo.Contains(client_control) Then
            ' We have RangeInfo for this control. Return it.
            Return DirectCast(m_RangeInfo(client_control), RangeInfo)
        Else
            ' We do not have RangeInfo for this control.
            ' Return a new default RangeInfo.
            Return New RangeInfo
        End If
    End Function

    ' Add or remove this RangeInfo if necessary.
    Private Sub AddOrRemoveIfNecessary(ByVal client_control As Control, ByVal range_info As RangeInfo)
        ' See if the RangeInfo should be present but is not,
        ' or should not be present but is.
        If range_info.IsDefault <> Not m_RangeInfo.Contains(client_control) Then
            If range_info.IsDefault Then
                ' The RangeInfo should not be present but is.
                m_RangeInfo.Remove(client_control)
                RemoveHandler client_control.Validating, AddressOf Client_Validating
            Else
                ' The RangeInfo should be present but is not.
                m_RangeInfo.Add(client_control, range_info)
                AddHandler client_control.Validating, AddressOf Client_Validating
            End If
        End If
    End Sub

    ' If some control has an error, display its error message,
    ' set focus to it, and return True. If all controls are
    ' okay, then return False.
    Public Function HasError() As Boolean
        If Not m_Enabled Then Return False

        ' Make sure all controls have been validated.
        Dim first_text_box As TextBox = Nothing
        Dim first_tab_index As Integer = Integer.MaxValue
        For Each text_box As TextBox In m_RangeInfo.Keys
            ' Validate the TextBox.
            ValidateTextBox(text_box)

            ' See if this is the first in the tab order so far.
            If errOutOfRange.GetError(text_box).Length > 0 Then
                If text_box.TabIndex < first_tab_index Then
                    first_text_box = text_box
                    first_tab_index = text_box.TabIndex
                End If
            End If
        Next text_box

        ' See if any control has an error.
        If Not (first_text_box Is Nothing) Then
            first_text_box.Focus()
            MessageBox.Show(errOutOfRange.GetError(first_text_box))
            Return True
        End If

        Return False
    End Function
End Class
